# Copyright (c) 2017 Turbopay Holdings Ltd

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE

import requests
from requests.auth import HTTPBasicAuth
from atomicpay.equity import exceptions
from atomicpay.equity import routes


class Auth(object):

    def __init__(self, consumer_key, consumer_secret, username, passwd, live=False):

        self.consumer_key = str(consumer_key)
        self.consumer_secret = str(consumer_secret)
        self.username = username
        self.passwd = passwd

        self.live = live
        self._auth = None
        self.authed = False

    def auth(self):
        """Used generate an OAuth access token to access other APIs"""

        url = routes.TOKEN

        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        auth = HTTPBasicAuth(self.consumer_key, self.consumer_secret)

        data = {
            "username": self.username,
            "password": self.passwd,
            "grant_type": "password"
        }

        response = requests.post(url, data=data, headers=headers, auth=auth)

        if not response.ok:
            error = response.json()
            msg = " ".join([error['code'], error['error'], error['message']])
            raise exceptions.AuthenticationFailed(msg)

        data = response.json()
        self._auth = data
        self.authed = True
        return

    @property
    def access_token(self):
        if not self.authed:
            self.auth()
            return self._auth['access_token']
        return self._auth['access_token']

    @property
    def expires_in(self):
        if not self.authed:
            self.auth()
            return self._auth['expires_in']
        return self._auth['expires_in']
